import os
from itertools import product

import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from matplotlib import rcParams
import pyratbay.io as io


# Plot results:
cols = {
    'H2O':"blue",
    'CH4':"limegreen",
    'CO':"red",
    'CO2': "purple",
    'NH3':"magenta",
    'C2H2':"chocolate",
    'C2H4':"salmon",
    'HCN':"deepskyblue",
    'N2':"olive",
    'H2':"orange",
    'H':'k',
    'He':"royalblue",
    'CIA': 'deepskyblue',
    'Rayleigh':'blue',
    'alkali':'k',
    }

the_seven = 'H2O CH4 CO CO2 NH3 HCN C2H2'.split()

metal = np.logspace(-3, 2, 6)
co_ratio = [0.1, 0.55, 0.9, 0.95, 1.05, 1.5, 2.0, 5.0]

nmetal = len(metal)
nco_ratio = len(co_ratio)
nlayers = 56
nspec = 18
temps = np.zeros((nmetal, nco_ratio, nlayers))
abund = np.zeros((nmetal, nco_ratio, nspec, nlayers))

# Read the atmospheric files:
folder = 'run_radeq/'
for iz,ico in product(range(nmetal), range(nco_ratio)):
    z = metal[iz]
    co = co_ratio[ico]
    units, specs, press, temp, q, r = io.read_atm(
        f'{folder}HD209458b_radeq_{z:07.3f}x-solar_CO-{co:4.2f}.atm')
    temps[iz, ico] = temp
    abund[iz, ico] = q.T

# Setting the right font can be tricky, but this code won't break if not:
# 1. Fetch an Helvetica.ttf file (e.g., google it) and put it in this folder:
# from pathlib import Path
# print(Path(matplotlib.matplotlib_fname()).parent.joinpath('fonts/ttf'))
# 2. Then, you probably need to delete this file to refresh the fonts cache:
# print(matplotlib.font_manager._fmcache)
rcParams['font.family'] = 'sans-serif'
rcParams['font.sans-serif'] = ['Helvetica']
fontsize = 7
dpi = 100
dpi = 300


# Temperature:
plt.close(31)
plt.figure(31, (7.2, 5.4))
plt.clf()
plt.subplots_adjust(0.07, 0.08, 0.98, 0.96, wspace=0.05, hspace=0.28)
for iz in range(nmetal):
    ax = plt.subplot(2, 3, 1+iz)
    ax.tick_params(direction='in', labelsize=fontsize)
    ax.set_title(f'[M/H] = ${np.log10(metal[iz])}$', fontsize=fontsize)
    for ico in range(nco_ratio):
        plt.semilogy(
            temps[iz,ico], press, lw=1.0,
            c=plt.cm.viridis_r(ico/nco_ratio),
            label=f'C/O = {co_ratio[ico]}')
    plt.ylim(np.amax(press), np.amin(press))
    plt.ylim(np.amax(press), 1e-8)
    plt.xlim(600, 3100)
    if iz%3 == 0:
        plt.ylabel("Pressure (bar)", fontsize=fontsize)
    else:
        ax.set_yticklabels([])
    if iz == nmetal-1:
        plt.legend(loc='upper right', fontsize=fontsize-0.5)
    plt.xlabel("Temperature (K)", fontsize=fontsize)
    ax.set_xticks([1000, 2000, 3000])
    ax.get_xaxis().set_major_formatter(
        matplotlib.ticker.FuncFormatter(lambda x, p: format(int(x), ',')))

plt.savefig('plots/HD209458b_radiative_equilibrium_temperature.pdf')
plt.savefig('plots/HD209458b_radiative_equilibrium_temperature.eps')
# For jpg outputs you need to have pillow installed (pip install pillow)
plt.savefig('plots/HD209458b_radiative_equilibrium_temperature.jpg', dpi=dpi)


# Mol mixing fraction abundances:
nco = 7
plt.close(41)
plt.figure(41, (7.2, 8.5))
plt.clf()
plt.subplots_adjust(0.07, 0.05, 0.9, 0.975, hspace=0.05, wspace=0.05)
for iz,ico in product(range(nmetal), range(nco)):
    ax = plt.subplot(nco, nmetal, ico*nmetal + iz + 1)
    ax.tick_params(direction='in', labelsize=fontsize)
    for q, spec in zip(abund[iz,ico], specs):
        if spec not in cols:
            continue
        lw = 1.0 if spec in the_seven else 0.5
        z = 100 if spec in the_seven else 1.0
        dashes = () if spec in the_seven else (8,3)
        plt.loglog(
            q, press, label=spec, lw=lw, c=cols[spec], dashes=dashes,
            zorder=z)
    plt.ylim(np.amax(press), np.amin(press))
    plt.ylim(np.amax(press), 1e-8)
    plt.xticks(np.logspace(-16, 0, 5))
    plt.xticks(np.logspace(-10, 0, 3))
    plt.xlim(1e-14, 3.0)
    if iz == 0:
        plt.ylabel("Pressure (bar)", fontsize=fontsize)
    else:
        ax.set_yticklabels([])
    if iz == nmetal-1:
        ax.text(
            1.04, 0.5, f"C/O = ${co_ratio[ico]}$", fontsize=fontsize,
            transform=ax.transAxes, va='center', rotation='vertical')
    if ico == nco-1:
        ax.set_xlabel('Mol fraction', fontsize=fontsize)
    else:
        ax.set_xticklabels([])
    if ico == 0:
        ax.set_title(f'[M/H] = ${np.log10(metal[iz])}$', fontsize=fontsize)
    if ico == 0 and iz == nmetal-1:
        leg = plt.legend(
            fontsize=fontsize-0.5, bbox_to_anchor=(1.17, 1),
            loc='upper left', borderaxespad=0.0, handlelength=1.0)

plt.savefig('plots/HD209458b_radiative_equilibrium_abundances.pdf')
plt.savefig('plots/HD209458b_radiative_equilibrium_abundances.eps')
plt.savefig('plots/HD209458b_radiative_equilibrium_abundances.jpg', dpi=dpi)

